#!/usr/bin/env python
import os
import math
import json
import time
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.ticker as tic
from utils import *

from multiprocessing import Pool
import sys, getopt

# loss option 'l2' or 'lr'
loss = 'lr'
# above line, True or False
above = True
ef = 'final'

x_label = 'CG'
# fig type
fig_type = 'eps'
# stopping
# 0, 1, 2, 3, 4 -> 1e-1, 1e-2, 1e-3, 1e-4, 1e-5
stopping_value = 4

S_Key = 'f'
# solver option
linestyles_l = ['--']
markers = ['^','v','p','D','o']
colors=['blue','m','tomato','k','g']
linestyles_l = ['--', '--', '--', '--']
datasets = [
  "news20.binary",
  #"w8a",
  #'covtype',
  #"rcv1_test",
  #"yahoojp",
  #"yahookr",
  #"url_combined",
  #"kdda",
  #"kddb",
  #"criteo.trva",
  #"kdd12.svm",
]
es = 1e-10

use_best = True
if use_best:
  DIR = 'log_{}_best/'.format(loss)
  REF_DIR = 'ref_log_{}_best/'.format(loss)
  if ef == 'l2_time':
    DIR = 'time_{}_best/'.format(loss)
  if ef == 'final_time':
    DIR = 'time_{}_best/'.format(loss)  
  if ef == 'final_1':
    DIR = 'time_{}_best/'.format(loss)  
else:
  DIR = 'log_{}/'.format(loss)
  REF_DIR = 'ref_log_{}/'.format(loss)
  cs = [0.01, 1, 100]

FIGURE_DIR = '{}_{}_ex'.format(loss, x_label)

print(datasets)

def zero_fix(x):
  if x < 1e-10: x = 1e-10
  return x

def plot_one(args):
  data, c, itr = args
  print('Drawing Results for {}, C = {}...'.format(data, c))

  start = time.time() 
  fig, ax = plt.subplots()
  if S_Key == 'f':
    ref_f_line = ref_f(file_name(data, 'origin', c, 1e-10, REF_DIR))

    for solver in solvers:
      #print(solver)
      ref_now = ref_f(file_name(data, solver, c, 1e-10, DIR))
      if ref_now < 1e-18: continue
      #print('done', solver)
      #if ref_now < ref_f_line:
      #  ref_f_line = ref_now
    # use tron to draw lines
    lcolors = ['k', 'm', 'y']
    line_data = ['origin']

    linestyles_stop = ['-', '--', '-.', ':']
    for j, solver in enumerate(line_data):
      for i, eps_line in enumerate(get_lines(data, solver, c, REF_DIR)[:stopping_value]):
        ax.axhline(
            y=(eps_line - ref_f_line) / ref_f_line,
            color=lcolors[j], 
            linestyle=linestyles_stop[i % stopping_value],
            )
    e5_line = get_lines(data, line_data[0], c, DIR)[stopping_value]
    e5_line = (e5_line-ref_f_line)/ref_f_line

    for i, solver in enumerate(solvers):
      X,Y = dataof(file_name(data, solver, c, es, DIR), S_Key, x_label=x_label)
      X ,Y = zip(*[(x,(float(y)-ref_f_line)/ref_f_line) for x,y in zip(X,Y) if float(y) > ref_f_line])

      if not above: Y = [zero_fix(y) for y in Y]
      args = {
          'linestyle' : linestyles_l[i % len(linestyles_l)], # linestyles[i % 4],
          'marker' : markers[i],
          'color' : colors[i],
          'linewidth' : 3,
          'markersize': 13,
          #'markerfacecolor': 'none',
          'markeredgewidth': 0,
          'markevery' : (len(X)+20) // 20
      }

      l_label = legend_label(solver=solver, ef=ef)
      line, = ax.plot(X, Y, label = l_label, **args)

    # set ylabel
    ax.set_ylabel('$('+S_Key+' - f^{*}) / f^{*}$', fontsize=23)
    # set eps = 1e-5 line as bottom
    if e5_line > 0 and above: ax.set_ylim(bottom=e5_line)
  elif S_Key == 'delta':
    for i, solver in enumerate(solvers):
      X,Y = dataof(file_name(data, solver, c, es, DIR), S_Key, x_label=x_label)
      args = {
          'linestyle' : linestyles_l[i % len(linestyles_l)], # linestyles[i % 4],
          'marker' : markers[i],
          'color' : colors[i],
          'linewidth' : 3,
          'markersize': 13,
          #'markerfacecolor': 'none',
          'markeredgewidth': 0,
          'markevery' : 5 #(len(X)+10) // 10
      }
      l_label = legend_label(solver=solver, ef=ef)
      line, = ax.plot(X, Y, label = l_label, **args)
    ax.set_ylabel('$\Delta_k$', fontsize=23)

  ax.set_yscale('log')
  formatter = tic.ScalarFormatter()
  formatter.set_scientific(True)
  formatter.set_powerlimits((0,0))
  ax.xaxis.set_major_formatter(formatter)
  plt.rc('font', size=15)
  for tick in ax.xaxis.get_major_ticks():
    tick.label.set_fontsize(15)
  for tick in ax.yaxis.get_major_ticks():
    tick.label.set_fontsize(15)
  if x_label == 'time':
    ax.set_xlabel('Training time (sec)', fontsize=23)
  elif x_label == 'Datapass':
      ax.set_xlabel('Datapass', fontsize=23)
  else:
    ax.set_xlabel('CG steps', fontsize=23)

  ax.legend(fontsize=20, loc='upper right')
  if loss == 'lr':
    if x_label == 'CG':
      is_set = False
      if data == 'url_combined':
        if c in (1,100): is_set = True
        if c == 1: ax.set_xlim([0, 3.5e2])
        elif c == 100: ax.set_xlim([0, 1e3])
      elif data == 'news20.binary':
        if c in (1,100): is_set = True
        if c == 1: ax.set_xlim([0, 3e2])
        elif c == 100: ax.set_xlim([0, 8e2])
      elif data == 'yahookr':
        if c in (1,100): is_set = True
        if c == 1: ax.set_xlim([0, 9e2])
        elif c == 100: ax.set_xlim([0, 4.5e3])
      elif data == 'kddb':
        if c in (1,100): is_set = True
        if c == 1: ax.set_xlim([0, 3.5e3])
        elif c == 100: ax.set_xlim([0, 3e4])
      elif data == 'criteo.trva':
        if c in (1,100): is_set = True
        if c == 1: ax.set_xlim([0, 1e2])
        elif c == 100: ax.set_xlim([0, 6e2])
      elif data == 'kdd12.svm':
        if c in (1,100): is_set = True
        if c == 1: ax.set_xlim([0, 4e2])
        elif c == 100: ax.set_xlim([0, 1e4])
      if ef in ('diag_m', 'diag_p') and not is_set:
        e5_ref = get_lines(data, line_data[0], c, DIR)[stopping_value]
        X,Y = dataof(file_name(data, 'origin', c, es, DIR), S_Key, x_label=x_label, ref_f=e5_ref)
        print('last CG = {:.2e}'.format(X[-1]))
        xlim = X[-1] * 1.1
        ax.set_xlim([0, xlim])
        X,Y = dataof(file_name(data, 'origin', c, es, DIR), S_Key, x_label=x_label)
        print('last CG (no e5) = {:.2e}'.format(X[-1]))
    # label == time
    elif data == 'criteo.trva':
      if c == 1: ax.set_xlim([0, 1.2e3])
      elif c == 100: ax.set_xlim([0, 6e3])
    elif data == 'w8a':
      if c == 1 and x_label == 'time':
        #ax.set_ylim(top=100)
        ax.set_xlim([0, 2])
        ax.legend(fontsize=20, loc='lower right')
      if c == 100:
        if x_label == 'time':
          ax.set_ylim(top=100)
          ax.set_xlim([0, 2])
          ax.legend(fontsize=20, loc='lower right')
          #ax.set_xscale('log')
        if x_label == 'CG':
          ax.set_ylim(top=100)
          ax.set_xlim([0, 1500])
          ax.legend(fontsize=20, loc='lower right')
          #ax.set_xscale('log')
    if data == 'news20.binary':
      if c == 1 and x_label == 'time':
        ax.set_xlim([0, 100])
        ax.legend(fontsize=20, loc='lower right')
      if c == 100 and x_label == 'time':
        ax.set_xlim([0, 100])
        ax.legend(fontsize=20, loc='lower right')
    if ef == 'subhessian_rand':
      ax.legend(fontsize=20, loc='lower right')
    if ef == 'subhessian_rand_delta':
      ax.legend(fontsize=20, loc='lower right')
      ax.set_xlim([0, 6000])

  dataname = data2name(data)
  dataname_out = dataname.replace('_', '-')
  dataname_out = dataname_out.replace('.', '-')
  if use_best:
    figure_root = 'paper'
  else:
    figure_root = 'paper_nb'
  if above:
    if ef != 'alpha':
      dr = '{}/{}_{}_best'.format(figure_root, ef, loss)
      check_and_create(dr) 
      figure_name = '{}/{}_{}_{:.0e}.{}'.format(
          dr, dataname_out, itr, c, fig_type)
    else:
      dr = '{}/{}_{}_{}_best'.format(figure_root, ef, x_label, loss)
      check_and_create(dr)
      figure_name = '{}/{}_{}_{:.0e}.{}'.format(
          dr, dataname_out, itr, c, fig_type)
  elif ef != 'alpha':
    dr = '{}/{}_{}'.format(figure_root, ef, loss)
    check_and_create(dr)
    figure_name = '{}/{}_{}.{}'.format(dr, dataname, str(c), fig_type)
  else:
    dr = '{}/{}_{}_{}'.format(figure_root, ef, x_label, loss)
    check_and_create(dr)
    figure_name = '{}/{}_{}.{}'.format(dr, dataname_out, str(c), fig_type)

  plt.tight_layout()
  plt.savefig(figure_name,bbox_inches='tight')
  plt.close()
  print('spent {:.3f} tims'.format((time.time()-start)))

def opt_parse(opts):
  global ef
  global solvers
  global cs
  global x_label
  for opt in opts:
    if opt[0] == '-e':
      if opt[1] == '1':
          print('Plotting Experiment 1')
          ef = 'exp1'
          solvers = [
            'origin',
            'diag_no_eet',
            'diag_mul_no_eet',
            'diag_parallel'
          ]
          cs = [1, 100]
          x_label = "CG"
      elif opt[1] == '2':
          print('Plotting Experiment 2')
          ef = 'exp2'
          solvers = [
            'origin',
            'woodbury_100',
            'woodbury_1000',
            'woodbury_3000'
          ]
          cs = [1, 100]
          x_label = "CG"
      elif opt[1] == '3':
          print('Plotting Experiment 3')
          ef = 'exp3'
          solvers = [
            'origin',
            'diag_no_eet',
            'diag_mul_no_eet',
            'woodbury_3000'
          ]
          cs = [1, 100]
          x_label = "CG"
      elif opt[1] == '4':
          print('Plotting Experiment 3 (Running Time)')
          ef = 'exp3_time'
          solvers = [
            'origin',
            'diag_no_eet',
            'diag_mul_no_eet',
            'woodbury_3000'
          ]
          cs = [1, 100]
          x_label = "time"

def init():
  try:
    opts, args = getopt.getopt(sys.argv[1:],"e:")
  except getopt.GetoptError as err:
    print(str(err))
    exit(2)
  opt_parse(opts)

if __name__ == '__main__':
  init()
  args = []
  #P = Pool(8)

  for data in datasets:
    for itr, c in enumerate(cs):
      args.append((data,c, itr))
  #P.map(plot_one, args)
  for arg in args: plot_one(arg)
  # plot_one(data, c)
